# Copyright (c) 1999-2003 Xerox Corporation.  All Rights Reserved.
#
# Permission to use, copy, modify  and  distribute  without  charge
# this  software,  documentation, images, etc. is granted, provided
# that this copyright and the author's name is retained.
#    
# A fee may be charged for this program ONLY to recover  costs  for
# distribution  (i.e.  media costs).  No profit can be made on this
# program.
#        
# The author assumes no responsibility for  disasters  (natural  or
# otherwise) as a consequence of use of this software.
#            
# Adam Stein (adam\@scan.mc.xerox.com)

# Open a connection to a CDE calendar
sub OpenCalendar {
  my ($cal) = shift,				# Calendar to open
  my ($session) = shift;			# Handle to calendar

  if ($USECSA == 1) {
    eval {
      $$session = Calendar::CSA::logon("",
				       {
				         'user_name' => $cal,
				         'user_type' => 'INDIVIDUAL',
				         'calendar_address' => $cal,
				       })
    };

    if ($$session) {
      $$session->short_entry_names(1);
      $$session->unix_times(1);
    }
  } else {
    $$session = $cal;
  }

  return($@);
}

# Close a connection to a CDE calendar
sub CloseCalendar {
  my ($session) = shift;			# Handle to calendar

  if ($USECSA == 1) {
    eval { $session->logoff; };

    if ($@) {
      print "<P>Error $@ while logging off of calendar</P>";
    }
  }
}

# Get appointments from the calendar
sub GetAppts {
  my ($session) = shift,			# Handle to calendar
  my ($day) = shift,				# Day to fetch appts for
  my ($appts) = shift,				# List of appts to return
  my ($id) = shift,				# Appointment reference IDs
  my ($cut) = shift;				# Flag to cut down appt

  if ($USECSA == 1) {
    return(&GetAppts_CSA($session, $day, $appts, $id, $cut));
  } elsif ($USECSA == 0) {
    return(&GetAppts_NonCSA($session, $day, $appts, $cut));
  }
}

# Fetch appointments using the CSA library
sub GetAppts_CSA {
  my ($session) = shift,			# Handle to calendar
  my ($day) = shift,				# Day to fetch appts for
  my ($appts) = shift,				# List of appts to return
  my ($id) = shift,				# Appointment reference IDs
  my ($repeat) = shift,				# True if appointment repeats
  my ($cut) = shift,				# Flag to cut down appt
  my ($appt),					# Full appointment (w/ time)
  my (@curday),					# Current day
  my ($end),					# End time of event
  my (@endtime),				# Ending time of appt
  my ($entrylist),				# Calendar appointments
  my ($isendtime),                            	# Is there Ending time of appt
  my (@nextday),				# Day after date
  my ($offset),					# Offset (in sec) from UTC
  my ($start),					# Start time of event
  my (@starttime),				# Starting time of appt
  my ($val1),					# Value of current day
  my ($val2);					# Value of next day

  # Find the offset (in seconds) from UTC to our timezone (the
  # CSA routine only takes time in UTC)
  $offset = -&tz_local_offset();

  # Find our starting date/time for our timezone of today's date at 00:00:00
  @curday = Add_Delta_DHMS($$day[2], $$day[0], $$day[1], 0, 0, 0, 0, 0, 0,
			   $offset);

  # Find day after's date so we can search for all dates between
  # the beginning of the day we want and the next day
  @nextday = Add_Delta_DHMS(@curday, 1, 0, 0, 0);

  # Put together values needed to fetch appointments
  $val1 = sprintf "%4d%02d%02dT%02d%02d%02dZ", $curday[0], $curday[1], $curday[2], $curday[3], $curday[4], $curday[5];
  $val2 = sprintf "%4d%02d%02dT%02d%02d%02dZ", $nextday[0], $nextday[1], $nextday[2], $nextday[3], $nextday[4], $nextday[5];

  eval {
  # Fetch appointments.  The format for the value is "YYYYMMDDTHHMMSSZ".
  # I have no idea what the 'T' or 'Z' letters are for.  The time is UTC.
  $entrylist = $session->list_entries(
	"Start Date" => {
	  type => "DATE TIME",
	  value => "$val1",
	  match => "GREATER THAN OR EQUAL TO"
	},

	"Start Date" => {
	  type => "DATE TIME",
	  value => "$val2",
	  match => "LESS THAN"
	}
  );
  };

  return(0) if ($@);

  if (defined($entrylist)) {
    foreach ($entrylist->entries) {
      # Get the author of the calendar entry so that we can add it to the
      # appointment (useful for common group calendars as opposed to an
      # individual's calendar)
      $organizer = $_->read_entry_attributes('Organizer')->{value}{user_name};
      ($author = $organizer) = ~s/@.+//;
      $author = (defined($calendar{$author}) ? $calendar{$author}[0] : $organizer);
      if ((&IsTimeless($_->read_entry_attributes('Start Date')->{value})) ||
	  ($_->read_entry_attributes('Show Time')->{value} == 0)) {
	# Append author of calendar entry to appointment
	$appt = $_->read_entry_attributes('Summary')->{value} .
	        "<BR><EM>(Author: $author)</EM><BR>";
      } else {
	($start = scalar(localtime($_->read_entry_attributes('Start Date')->{value}))) =~ s/^... ... .. (..:..).*/$1/;

	# may not be an end time - pjmp
	if ( eval {
	  ($end = scalar(localtime($_->read_entry_attributes('End Date')->{value}))) =~ s/^... ... .. (..:..).*/$1/ if (!$cut);
	     } ) {
	  # OK
	  $isendtime = 1;
	} else {
	  # Not OK
	  $isendtime = 0;
	}

	# Get start & end hours to check for am/pm
	@starttime = split(/:/, $start);
	@endtime = split(/:/, $end) if (!$cut);

	# Set up times
	if ($starttime[0] == 0) {		# Midnight
	  $starttime[0] = 12;
	  $starttime[2] = "am";
	} elsif ($starttime[0] < 12) {		# Morning
	  $starttime[2] = "am";
	} elsif ($starttime[0] == 12) {		# Noon
	  $starttime[2] = "pm";
	} else {				# Afternoon/Evening
	  $starttime[0] -= 12;
	  $starttime[2] = "pm";
	}

	if (!$cut && $isendtime) {
	  if ($endtime[0] == 0) {		# Midnight
	    $endtime[0] = 12;
	    $endtime[2] = "am";
	  } elsif ($endtime[0] < 12) {		# Morning
	    $endtime[2] = "am";
	  } elsif ($endtime[0] == 12) {		# Noon
	    $endtime[2] = "pm";
	  } else {				# Afternoon/Evening
	    $endtime[0] -= 12;
	    $endtime[2] = "pm";
	  }
	}

	# Put together appointment
	if (!$cut && $isendtime) {
	  # Append author of calendar entry to appointment
	  $appt = "$starttime[0]:$starttime[1]$starttime[2]-" .
		  "$endtime[0]:$endtime[1]$endtime[2] " .
	          $_->read_entry_attributes("Summary")->{value} .
		  "<BR><EM>(Author: $author)</EM><BR>";
	} elsif ( !$cut ) {
	  # Append author of calendar entry to appointment
	  $appt = "$starttime[0]:$starttime[1]$starttime[2] " .
		  $_->read_entry_attributes("Summary")->{value} .
		  "<BR><EM>(Author: $author)</EM><BR>";
	} else {
	  $appt = "$starttime[0]:$starttime[1]$starttime[2] " .
		  $_->read_entry_attributes("Summary")->{value};
	}
      }

      # Limit the length, convert spaces to non-breaking spaces
      $appt = substr($appt, 0, MAXCHAR) if ($cut);
      $appt =~ s/ /&nbsp;/g;

      # Store reference ID and recurrence rule (used for deletion)
      $$id{$appt} = $_->read_entry_attributes("Reference Identifier")->{value};
      $$repeat{$appt} = (defined($_->read_entry_attributes('Recurrence Rule')))
			? 1 : 0;

      push(@$appts, $appt);
    }
  }

  return(1);
}

# Is the appointment timeless?
sub IsTimeless {
  my ($tick) = shift,				# Time to check
  my (@time) = localtime($tick);		# Convert to localtime

  # Palm sets time to 3:41 am if it's a timeless event
  return ($time[2] == 3 &&
	  $time[1] == 41 &&
	  $time[0] == 0);
}

# Fetch appointments with the CDE lookup program
sub GetAppts_NonCSA {
  my ($session) = shift,			# Handle to calendar
  my ($day) = shift,				# Day to fetch appts for
  my ($appts) = shift,				# List of appts to return
  my ($cut) = shift,				# Flag to cut down appt
  my ($access) = 1,				# Indicates access to appts
  my ($appt),					# Appointment
  my ($date),					# Date to get appts for
  my ($lastappt);				# Appt from last line

  # Make sure variables sent to the shell are OK
  $session =~ tr/-[a-zA-Z0-9]_\@\.//cd;

  # Try to connect to the calendar
  $date = "$$day[0]/$$day[1]/$$day[2]";
  open(CAL, LOOKUP . " -c $session -v day -d $date 2>&1 |") ||
	die "$program: can't run [" . LOOKUP . "]\n";

  # Parse out appointments
  while (<CAL>) {
    # Return a "no access" status if we can't get to the appointments
    $access = 0, last if (/Could not open calendar/);

    if ($cut) {
      # If we have an appointment, remove the ending time (if there
      # is one), cut it down to fit the day box and add it to the list
      if (/^.*[0-9]\)\s*(.*)/) {
        $appt = $1;
        $appt =~ s/-\s*[0-9]+:[0-9][0-9][ap]m//;	# Remove ending time

	# Limit the length, convert spaces to non-breaking spaces
        $appt = substr($appt, 0, MAXCHAR);
        $appt =~ s/ /&nbsp;/g;

        # Store
        push(@$appts, $appt);
      }
    } else {
      $appt = $_;

      # Ignore the "Appointments for..." line
      next if ($appt =~ /^Appointments for/);

      # Ignore blank lines
      next if ($appt =~ /^\n/);

      # Chop off #) from front of appointment
      if (/^.*[0-9]\)\s*(.*)/) {
	$appt = $1;

	if (defined($lastappt)) {
          # Convert spaces to non-breaking spaces
	  $lastappt =~ s/ /&nbsp;/g;

          # Store
          push(@$appts, $lastappt);
	}

	$lastappt = $appt;
      } else {
	# Emphasis the "No Appointments..." message if we have it
	$appt = "<EM>$appt</EM>" if ($appt =~ s/^(No Appointments).*/$1/);

	$lastappt .= "<BR>$appt";
      }
    }
  }

  close(CAL);

  # Store last appointment only when saving ALL the lines
  # (cutting only saves the first line)
  if (!$cut) {
    # Convert spaces to non-breaking spaces
    $lastappt =~ s/ /&nbsp;/g;

    # Store
    push(@$appts, $lastappt);
  }

  return($access);
}

# Determine if we have sufficient access rights to insert/delete appointments.
# Return values are AND'd together.  Return 0x01 for insert privileges.
# Return 0x02 for delete privileges.   Return 0 if user has access to neither.
sub AccessRights {
  my ($cgi) = shift,                            # CGI object
  my ($access),					# Calendar access permissions
  my ($caladdr),                                # Calendar addr to connect to
  my ($key),                                    # Key to calendar to display
  my ($login),					# Login name
  my ($session),                                # Handle to CDE calendar
  my ($status);					# Access status

  # Assume the user has at least viewing access (this will
  # be tested later when the calendar is actually accessed)
  $status = A_VIEW;

  # Check for insertion/deletion privileges ONLY if we use CSA
  if ($USECSA == 1) {
    # See if we were given a full calendar address or
    # if we need to look it up
    $key = $cgi->param("key");
    $caladdr = (($key =~ /@/) ? $key : $calendar{$key}[1]);

    # Open connection to calendar
    if (($caladdr ne "") && (&OpenCalendar($caladdr, \$session) eq "")) {
      # Get access permissions
      eval {
        $access = $session->read_calendar_attributes("Access List")->{value};
      };

      $login = getlogin || getpwuid($>);

      if (!$@) {
        foreach (@$access) {
	  # Only need to check permissions for this user
	  # OR if world - gives default in absence of user
	  if ($login eq $$_{"user"}->{"user_name"} ||
	      $$_{"user"}->{"user_name"} eq "world" ) {
	    if ($$_{"rights"}->{"OWNER RIGHTS"}) {
	      $status = A_INSERT | A_DELETE;
	    } elsif ($$_{"rights"}->{"INSERT PUBLIC ENTRIES"}) {
	      $status |= A_INSERT;
	    } elsif ($$_{"rights"}->{"VIEW CALENDAR ATTRIBUTES"}) {
	      $status |= A_DELETE;
	    }
	  }
        }
      }

      &CloseCalendar($session);
    }
  }

  return($status);
}

# This comes from PilotManager
sub data
{
    my ($type, $value) = @_;

    # Make sure that the integer values aren't represented
    # as a string.  If they go to the Calendar::CSA layer as
    # strings, CSA assumes that they are in CSA's ISO format
    # and won't convert them.
    #
    if ($type =~ /^(DATE TIME|SINT32|UINT32|)$/)
    {
        $value += 0;
    }

    return ({
              'type' => $type,
              'value' => $value,
            });
}

1;

