# Copyright (c) 1999-2003 Xerox Corporation.  All Rights Reserved.
#
# Permission to use, copy, modify  and  distribute  without  charge
# this  software,  documentation, images, etc. is granted, provided
# that this copyright and the author's name is retained.
#    
# A fee may be charged for this program ONLY to recover  costs  for
# distribution  (i.e.  media costs).  No profit can be made on this
# program.
#        
# The author assumes no responsibility for  disasters  (natural  or
# otherwise) as a consequence of use of this software.
#            
# Adam Stein (adam\@scan.mc.xerox.com)

# Write out common HTML tags for a month view calendar
sub StartMonthCalendar {
  my ($month) = shift,				# Month to show
  my ($calwidth) = shift,			# Width to set table
  my ($url) = shift,				# URL to use for links
  my ($frame) = shift,				# Frame to display content in
  my (@lastmonth),				# Last month's date
  my (@nextmonth),				# Next month's date
  my ($nurl),					# URL for next month
  my ($purl),					# URL for previous month
  my ($width);					# Width to set table cells

  # Figure out what each day cell's width should be
  $width = $calwidth / 7 if (defined($calwidth));

  if (defined($calwidth)) {
    # Figure out last and next month's date for URL links
    @lastmonth = Add_Delta_YMD(@$month, 0, -1, 0);
    @nextmonth = Add_Delta_YMD(@$month, 0, 1, 0);

    # Set up previous and next month URLs
    $purl = "$url&day=1&month=$lastmonth[1]&year=$lastmonth[0]&view=" . V_MONTH;
    $nurl = "$url&day=1&month=$nextmonth[1]&year=$nextmonth[0]&view=" . V_MONTH;

    print "<TABLE ALIGN=\"center\" BORDER=\"1\" WIDTH=\"$calwidth\">\n",

	  "<CAPTION><A HREF=\"javascript:update($frame, '$purl', 7)\">",
	  "<IMG SRC=\"" . IMGURL . "/prev.gif\" ALT=\"[Prev Month]\" ",
	  "ALIGN=\"middle\" WIDTH=\"30\" HEIGHT=\"45\" VSPACE=\"10\" ",
	  "BORDER=\"0\"></A><BIG><STRONG>", Month_to_Text($$month[1]),
	  " $$month[0]</STRONG></BIG><A ",
	  "HREF=\"javascript:update($frame, '$nurl', 7)\">",
	  "<IMG SRC=\"" . IMGURL . "/next.gif\" ALT=\"[Next Month]\" ",
	  "ALIGN=\"middle\" WIDTH=\"30\" HEIGHT=\"45\" BORDER=\"0\"></A>",
	  "</CAPTION>\n";
  } else {
    $url .= "&day=1&month=$$month[1]&year=$$month[0]&view=" . V_MONTH;

    print "<TABLE BORDER=\"1\">\n",

          "<CAPTION><BIG><STRONG>",
	  "<A HREF=\"javascript:update($frame, '$url', 7)\">",
	  Month_to_Text($$month[1]), " ", $$month[0], "</STRONG></BIG>",
	  "</CAPTION>\n";
  }

  if (defined($calwidth)) {
    print "<TR>\n",
	  "<TH WIDTH=\"$width\">Sun</TH>\n",
	  "<TH WIDTH=\"$width\">Mon</TH>\n",
	  "<TH WIDTH=\"$width\">Tue</TH>\n",
	  "<TH WIDTH=\"$width\">Wed</TH>\n",
          "<TH WIDTH=\"$width\">Thu</TH>\n",
	  "<TH WIDTH=\"$width\">Fri</TH>\n",
	  "<TH WIDTH=\"$width\">Sat</TH>\n",
	  "</TR>\n";
  } else {
    print "<TR><TH>Sun</TH><TH>Mon</TH><TH>Tue</TH><TH>Wed</TH>",
          "<TH>Thu</TH><TH>Fri</TH><TH>Sat</TH></TR>\n";
  }
}

# Create a month calendar (with no events displayed)
sub CreateMonth {
  my ($month) = shift,                          # Month to creat calendar for
  my ($url) = shift,                            # URL to use for links
  my ($frame) = shift,				# Frame to display content in
  my ($blank),					# What to put in non-days
  my ($day),                                    # Day of the month
  my ($daysinmonth),                            # Days in month
  my ($loop),					# Loop thru days
  my ($numdays),                                # Number of days to display
  my ($start),                                  # Month's starting day
  my (@today),                                  # Today's date
  my ($url2);                                   # Full URL

  @today = Today;

  # Create calendar (can't use Date::Calc's Calendar() function
  # because we want to start weeks on a Sunday and it starts on a Monday
  &StartMonthCalendar($month, undef, $url, $frame);

  # Starting day (where Sunday = 0, Saturday = 6)
  $start = Day_of_Week(@$month) % 7;

  # Number of days in the month
  $daysinmonth = Days_in_Month($$month[0], $$month[1]);

  # The number of days in the month to display (whether
  # they are actual calendar days, or blanks to fill up a week)
  #
  # For the Year view, we always want to display 6 weeks just to
  # keep a consistant look across the year.  For the "prev" and "next"
  # months, we want the minimum number.
  $numdays = ($frame == 2) ? 42 : &ceil(($daysinmonth + $start) / 7) * 7;

  # The blank cell, non-breaking space for Year view, empty string
  # for the "prev" & "next" month views
  $blank = ($frame == 2) ? "&nbsp;" : "";

  for($loop = 0,$day = 1;$loop < $numdays;++$loop) {
    # Print HTML table row tags when needed
    print "</TR>\n" if (!($loop%7) && $loop);
    print "<TR>" if (!($loop%7));

    if (($loop >= $start) && ($day <= $daysinmonth)) {
      # Set up day link
      $url2 = $url;
      $url2 .= "&day=$day&month=$$month[1]&year=$$month[0]&view=" . V_DAY;

      # Highlight the cell if it's today's date
      if (($$month[1] == $today[1]) && ($day == $today[2]) &&
	  ($$month[0] == $today[0])) {
	print td({align => "center", bgcolor => TODAY},
		 a({href => "javascript:update($frame, '$url2', 7)"}, $day));
      } else {
	print td({align => "center"},
		 a({href => "javascript:update($frame, '$url2', 7)"}, $day));
      }

      ++$day;
    } else {
      print td($blank);
    }
  }

  print "</TABLE>";
}

sub ceil {
  my ($val) = shift;				# Value to evaluate

  return(($val > int($val)) ? (int($val) + 1) : int($val));
}

1;

