# Copyright (c) 1999-2003 Xerox Corporation.  All Rights Reserved.
#
# Permission to use, copy, modify  and  distribute  without  charge
# this  software,  documentation, images, etc. is granted, provided
# that this copyright and the author's name is retained.
#    
# A fee may be charged for this program ONLY to recover  costs  for
# distribution  (i.e.  media costs).  No profit can be made on this
# program.
#        
# The author assumes no responsibility for  disasters  (natural  or
# otherwise) as a consequence of use of this software.
#            
# Adam Stein (adam\@scan.mc.xerox.com)

# Timezones for day view
my(@times) = (undef, "12a", "7a", "8a", "9a", "10a", "11a", "12p", "1p", "2p",
	      "3p", "4p", "5p", "6p", "7p", "8p", "11p");

# Show the calendar
sub ShowCalendar {
  my ($cgi) = shift,                            # CGI object
  my ($view) = shift,				# Calendar view
  my ($date) = shift,				# Date to show
  my ($key) = shift,				# Key to calendar to display
  my ($session) = shift;			# Handle to CDE calendar

  SWITCH: {
    &ShowDayView($cgi, $date, $key, $session), last
	if ($view == V_DAY);

    &ShowWeekView($cgi, $date, $key, $session), last
	if ($view == V_WEEK);

    &ShowMonthView($cgi, $date, $key, $session), last
	if ($view == V_MONTH);

    &ShowYearView($cgi, $date, $key, $session), last
	if ($view == V_YEAR);

    print $cgi->h1("Illegal view chosen");
  }
}

# Show the given date in "day" view
sub ShowDayView {
  my ($cgi) = shift,                            # CGI object
  my ($date) = shift,				# Date to show
  my ($key) = shift,				# Key to calendar to display
  my ($session) = shift,			# Handle to CDE calendar
  my ($access) = 1,                             # Indicates access to appts
  my (@atime),					# Starting time of appt
  my (@appts),                                  # Appointments
  my ($calname),                        	# Name belonging to calendar
  my ($durl),					# URL for deletion choice win
  my (%IDs),					# Appointment reference IDs
  my (@last),                                   # Last day's date
  my (@match1),					# Matching appts < :30
  my (@match2),					# Matching appts >= :30
  my (@next),                                	# Next day's date
  my ($nurl),                                   # URL for next day
  my ($purl),                                   # URL for previous day
  my (%repeat),					# 1 if appointment repeats
  my ($rowspan),				# Number of rows to span
  my (@stime),					# Starting time of time zone
  my ($time),					# Time zone
  my (@thisday),                                # Date to get appts for
  my (@today),                                  # Today's date
  my ($url);					# URL to use for links

  # See if we need to fetch the name from a full calendar address
  if ($key =~ /@/) {
    ($calname = $key) =~ s/@.*//;
  } else {
    $calname = $key;
  }

  # Perl seems to need to reference $date before it is used,
  # otherwise the value gets lost
  $date = $date;

  @today = Today;

  # Set up for the day
  $thisday[0] = $$date[1];
  $thisday[1] = $$date[2];
  $thisday[2] = $$date[0];

  $access = &GetAppts($session, \@thisday, \@appts, \%IDs, \%repeat, 0);

  # Message the user if we can't connect to the calendar
  if (!$access) {
    print "<SCRIPT>\n",
          "alert(\"WARNING: Can't connect to [$calname]\\n\\n",
          "Appointments will NOT be listed\");",
          "</SCRIPT>\n";
  }

  # Figure out last and next days's date for URL links
  @last = Add_Delta_Days(@$date, -1);
  @next = Add_Delta_Days(@$date, 1);

  # Set up previous and next day URLs
  $url = $cgi->url . "?key=$key";
  $durl = $cgi->url . "/f2";
  $purl = "$url&day=$last[2]&month=$last[1]&year=$last[0]&view=" . V_DAY;
  $nurl = "$url&day=$next[2]&month=$next[1]&year=$next[0]&view=" . V_DAY;

  if (($thisday[0] == $today[1]) && ($thisday[1] == $today[2]) &&
      ($thisday[2] == $today[0])) {
    print "<TABLE ALIGN=\"center\" BORDER=\"1\" BGCOLOR=\"" . TODAY . "\">\n";
  } else {
    print "<TABLE ALIGN=\"center\" BORDER=\"1\">\n";
  }
	
  print "<CAPTION>", $cgi->a({href => "javascript:update(2, '$purl', 7)"},
	                     "<IMG SRC=\"" . IMGURL . "/prev.gif\" ",
			     "ALT=\"[Prev Day]\" ALIGN=\"middle\" ",
			     "WIDTH=\"30\" HEIGHT=\"45\" VSPACE=\"10\" ",
	                     "BORDER=\"0\">"),
	"<BIG><STRONG>", Date_to_Text_Long(@$date), "</STRONG></BIG>",
	$cgi->a({href => "javascript:update(2, '$nurl', 7)"},
	        "<IMG SRC=\"" . IMGURL . "/next.gif\" ALT=\"[Next Day]\" ",
	        "ALIGN=\"middle\" WIDTH=\"30\" HEIGHT=\"45\" BORDER=\"0\">"),
	"</CAPTION>\n";

  foreach $time (@times) {
    if ($access) {
      undef(@match1);
      undef(@match2);

      if (defined($time)) {
	@stime = split("[ap]", $time);			# Get starting time
	$stime[1] = substr($time, -1, 1);		# AM or PM notation

	# Find appts that match current time zone
	foreach (@appts) {
	  # Skip appointments with no time
	  next if (/^[^0-9]/);

	  chomp;
	  ($atime[0] = $_) =~ s/:.*//s;			# Get starting hour
	  ($atime[1] = $_) =~ s/[0-9]+:[0-9][0-9](.).*/$1/s; # Get AM or PM
	  ($atime[2] = $_) =~ s/[0-9]+:([0-9][0-9]).*/$1/s;  # Get starting min

	  # Add to list if we match the starting hour and AM/PM designation
	  if (($stime[0] == $atime[0]) && ($stime[1] eq $atime[1])) {
	    # Add to 1st half of hour or 2nd half of hour list
	    if ($atime[2] >= 30) {
	      push(@match2, "<A HREF=\"javascript:deleteWin('$durl', '$key', '$IDs{$_}', $repeat{$_}, $mode)\">" .
		   "<IMG SRC=\"" . IMGURL . "/delete.gif\"" .
		   "ALT=\"[Delete Appointment]\"" .
		   "BORDER=\"0\"></A> $_");
	    } else {
	      push(@match1, "<A HREF=\"javascript:deleteWin('$durl', '$key', '$IDs{$_}', $repeat{$_}, $mode)\">" .
		   "<IMG SRC=\"" . IMGURL . "/delete.gif\"" .
		   "ALT=\"[Delete Appointment]\"" .
		   "BORDER=\"0\"></A> $_");
	    }
	  }
	}

	$rowspan = 2;
      } else {
	@match1 = grep(/^[^0-9]/, @appts);
	$time = "&nbsp;";	# To get the table to look right
	$rowspan = 1;
      }

      @match1 = ("&nbsp;") if (!scalar(@match1));
      @match2 = ("&nbsp;") if (!scalar(@match2));

      # On the hour appoints is in the first TR, on the 1/2 hour in
      # the second TR
      print $cgi->TR($cgi->td({rowspan => $rowspan}, "$time"),
		     $cgi->td({width => "500"}, join("<BR>", @match1)));

      print $cgi->TR($cgi->td(join("<BR>", @match2))) if ($rowspan == 2);
    } else {
      print $cgi->TR($cgi->td({rowspan => 2}, "$time"),
		     $cgi->td({width => "500"}, "&nbsp;")),

	    $cgi->TR($cgi->td("&nbsp;"));
    }
  }

  print "</TABLE>\n";
}

# Show the given date in "week" view
sub ShowWeekView {
  my ($cgi) = shift,                            # CGI object
  my ($date) = shift,				# Date to show
  my ($key) = shift,				# Key to calendar to display
  my ($session) = shift,			# Handle to CDE calendar
  my ($access) = 1,                             # Indicates access to appts
  my ($calname),                        	# Name belonging to calendar
  my ($durl),                                   # URL for deletion choice win
  my (%IDs),                                    # Appointment reference IDs
  my (@last),                               	# Last week's date
  my ($loop),					# Loop thru week
  my ($messaged) = 0,                           # Indicates user was messaged
  my (@next),                               	# Next week's date
  my (%repeat),                                 # 1 if appointment repeats
  my (@thisday),                                # Date to get appts for
  my (@today),                                  # Today's date
  my ($week);					# Week number

  # Consider the user messaged if there isn't a calendar handle
  $messaged = 1 if (! defined($session));

  # See if we need to fetch the name from a full calendar address
  if ($key =~ /@/) {
    ($calname = $key) =~ s/@.*//;
  } else {
    $calname = $key;
  }

  # Perl seems to need to reference $date before it is used,
  # otherwise the value gets lost
  $date = $date;

  @today = Today;

  # Need to find the first Sunday before the date given and
  # start from there
  @$date = Add_Delta_Days(Monday_of_Week(Week_of_Year(@$date)), -1)
	if(Day_of_Week(@$date) != 7);

  # Figure out last and next week's date for URL links
  @last = Add_Delta_Days(@$date, -7);
  @next = Add_Delta_Days(@$date, 7);

  # Set up previous and next week URLs
  $url = $cgi->url . "?key=$key";
  $durl = $cgi->url . "/f2";
  $purl = "$url&day=$last[2]&month=$last[1]&year=$last[0]&view=" . V_WEEK;
  $nurl = "$url&day=$next[2]&month=$next[1]&year=$next[0]&view=" . V_WEEK;

  $week = Week_Number(@$date);

  print "<TABLE ALIGN=\"center\" BORDER=\"1\">\n",

	"<CAPTION>", $cgi->a({href => "javascript:update(2, '$purl', 7)"},
			     "<IMG SRC=\"" . IMGURL . "/prev.gif\" ",
			     "ALT=\"[Prev Week]\" ALIGN=\"middle\" ",
			     "WIDTH=\"30\" HEIGHT=\"45\" VSPACE=\"10\" ",
			     "BORDER=\"0\">"),
	"<BIG><STRONG>Week #$week</STRONG></BIG>",
	$cgi->a({href => "javascript:update(2, '$nurl', 7)"},
		"<IMG SRC=\"" . IMGURL . "/next.gif\" ALT=\"[Next Week]\" ",
		"ALIGN=\"middle\" WIDTH=\"30\" HEIGHT=\"45\" BORDER=\"0\">"),
	"</CAPTION>\n";

  for($loop = 0;$loop < 7;++$loop) {
    undef(@appts);

    # Set up for the day
    $thisday[0] = $$date[1];
    $thisday[1] = $$date[2];
    $thisday[2] = $$date[0];

    $access = &GetAppts($session, \@thisday, \@appts, \%IDs, \%repeat, 0);

    # Need to update appointments with URL for deletion
    foreach (@appts) {
      $_ = "<A HREF=\"javascript:deleteWin('$durl', '$key', '$IDs{$_}', $repeat{$_})\"><IMG SRC=\"" . IMGURL . "/delete.gif\" ALT=\"[Delete Appointment]\" BORDER=\"0\"></A> $_";
    }

    if ($access) {
      if (($thisday[0] == $today[1]) && ($thisday[1] == $today[2]) &&
	  ($thisday[2] == $today[0])) {
        print $cgi->TR($cgi->td({width => "40", bgcolor => TODAY},
		       $cgi->strong(Day_of_Week_to_Text(Day_of_Week(@$date)),
				    Month_to_Text($$date[1]),
				    " $$date[2], $$date[0]")),
		       $cgi->td({width => "500", bgcolor => TODAY},
				"<BR>", join("<BR>", @appts)));
      } else {
        print $cgi->TR($cgi->td({width => "40"},
		       $cgi->strong(Day_of_Week_to_Text(Day_of_Week(@$date)),
				    Month_to_Text($$date[1]),
				    " $$date[2], $$date[0]")),
		       $cgi->td({width => "500"}, "<BR>", join("<BR>", @appts)));
      }
    } elsif (!$messaged) {
      # Message the user if we can't connect to the calendar
      print "<SCRIPT>\n",
	    "alert(\"WARNING: Can't connect to [$calname]\\n\\n",
	    "Appointments will NOT be listed\");",
	    "</SCRIPT>\n";

      $messaged = 1;
    }

    @$date = Add_Delta_Days($thisday[2], $thisday[0], $thisday[1], 1);
  }

  print "</TABLE>";
}

# Show the given date in "month" view
sub ShowMonthView {
  my ($cgi) = shift,                            # CGI object
  my ($date) = shift,				# Date to show
  my ($key) = shift,				# Key to calendar to display
  my ($session) = shift,			# Handle to CDE calendar
  my ($access) = 1,                             # Indicates access to appts
  my (@appts),                                  # Appointments
  my ($calname),                        	# Name belonging to calendar
  my ($days),                                   # Days in month
  my (%IDs),                                    # Appointment reference IDs
  my ($messaged) = 0,                           # Indicates user was messaged
  my (%repeat),                                 # 1 if appointment repeats
  my ($start),                                  # Previous month's starting day
  my (@thisday),                                # Date to get appts for
  my (@today),					# Today's date
  my ($url);					# URL to use for links

  # Consider the user messaged if there isn't a calendar handle
  $messaged = 1 if (! defined($session));

  # See if we need to fetch the name from a full calendar address
  if ($key =~ /@/) {
    ($calname = $key) =~ s/@.*//;
  } else {
    $calname = $key;
  }

  # Perl seems to need to reference $date before it is used,
  # otherwise the value gets lost
  $date = $date;

  @today = Today;

  # Figure out the days for the previous month and the starting day
  $days = Days_in_Month($$date[0], $$date[1]);
  $start = Day_of_Week($$date[0], $$date[1], 1);

  # Create URL for links in this frame
  $url = $cgi->url . "?key=" . $cgi->param("key");

  # Create the calendar showing off the month's entries
  &StartMonthCalendar($date, CALWIDTH, $url, 2);

  # Set up for first day in month
  $thisday[0] = $$date[1];
  $thisday[1] = 1;
  $thisday[2] = $$date[0];

  for($counter = 0,$day = 1;$day <= $days;++$counter) {
    print "</TR>\n" if (!($counter%7) && $counter);
    print "<TR>" if (!($counter%7));

    if ($counter >= $start) {
      # Get appointments for the day
      $access = &GetAppts($session, \@thisday, \@appts, \%IDs, \%repeat, 1)
	if ($access);

      # Message the user that we can't connect to the calendar
      if (!$access && !$messaged) {
        print "<SCRIPT>\n",
	      "alert(\"WARNING: Can't connect to [$calname]\\n\\n",
	      "Appointments will NOT be listed\");",
	      "</SCRIPT>\n";

        $messaged = 1;
      }

      if (scalar(@appts)) {
	$url = $cgi->url . "?key=$key&day=$day&month=$$date[1]&" .
	       "year=$$date[0]&view=" . V_DAY;

	if (($thisday[0] == $today[1]) && ($thisday[1] == $today[2]) &&
	    ($thisday[2] == $today[0])) {
          print "<TD VALIGN=\"top\" BGCOLOR=\"" . TODAY . "\"><STRONG>",
		"<A HREF=\"javascript:update(2, '$url', 5)\">$day</A>",
		"</STRONG><BR>";
	} else {
          print "<TD VALIGN=\"top\"><STRONG>",
	        "<A HREF=\"javascript:update(2, '$url', 5)\">$day</A>",
		"</STRONG><BR>";
	}

	foreach (@appts) {
	  print "<BR><FONT SIZE=\"2\">$_</FONT>\n";
	}

	print "</TD>\n";
      } elsif (($thisday[0] == $today[1]) && ($thisday[1] == $today[2]) &&
	       ($thisday[2] == $today[0])) {
	print "<TD VALIGN=\"top\" BGCOLOR=\"" . TODAY . "\"><STRONG>$day</STRONG>",
	      "</TD>\n";
      } else {
	print "<TD VALIGN=\"top\"><STRONG>$day</STRONG></TD>\n";
      }

      ++$day;

      # Set up for next day appointments
      ++$thisday[1];
      undef(@appts);
    } else {
      print "<TD>&nbsp;</TD>";
    }
  }

  # Finish off the rest of the cells in this row if there are any
  while ($counter%7) {
    print "<TD>&nbsp;</TD>";
    ++$counter;
  }

  print "</TR>" if ($counter%7);

  print "</TABLE>\n";
}

# Show the given date in "year" view
sub ShowYearView {
  my ($cgi) = shift,                            # CGI object
  my ($date) = shift,				# Date to show
  my ($key) = shift,				# Key to calendar to display
  my ($session) = shift,			# Handle to CDE calendar
  my (@last),					# Last year's date
  my ($loop),					# Loop thru months
  my (@next),					# Next year's date
  my ($nurl),                                   # URL for next day
  my ($purl),                                   # URL for previous day
  my ($url);					# URL to use for links

  # Perl seems to need to reference $date before it is used,
  # otherwise the value gets lost
  $date = $date;

  # Figure out last and next year's date for URL links
  @last = Add_Delta_YMD(@$date, -1, 0, 0);
  @next = Add_Delta_YMD(@$date, 1, 0, 0);

  # Create URL for links in this frame
  $url = $cgi->url . "?key=" . $cgi->param("key");

  # Set up previous and next day URLs
  $purl = "$url&day=$last[2]&month=$last[1]&year=$last[0]&view=" . V_YEAR;
  $nurl = "$url&day=$next[2]&month=$next[1]&year=$next[0]&view=" . V_YEAR;

  # Set to first month in year
  $$date[1] = 1;

  # Need to put the individual month tables in a bigger table to
  # get 3 months on the same line
  print "<TABLE><TR>",
	"<CAPTION>", $cgi->a({href => "javascript:update(2, '$purl', 7)"},
			     "<IMG SRC=\"" . IMGURL . "/prev.gif\" ",
			     "ALT=\"[Prev Year]\" ALIGN=\"middle\" ",
			     "WIDTH=\"30\" HEIGHT=\"45\" VSPACE=\"10\" ",
			     "BORDER=\"0\">"),
	"<BIG><STRONG>$$date[0]</STRONG></BIG>",
	$cgi->a({href => "javascript:update(2, '$nurl', 7)"},
		"<IMG SRC=\"" . IMGURL . "/next.gif\" ALT=\"[Next Year]\" ",
		"ALIGN=\"middle\" WIDTH=\"30\" HEIGHT=\"45\" BORDER=\"0\">"),
	"</CAPTION>\n";

  # Set the starting month to 1 to start with January.
  # Set the day to 1 since every month is guaranteed to have a day 1
  $$date[1] = $$date[2] = 1;

  # Create a year's worth of months
  for($loop = 0;$loop < 12;++$loop) {
    print "</TR>\n" if (!($loop%3) && $loop);
    print "<TR>" if (!($loop%3));

    print "<TD VALIGN=\"top\">";
    &CreateMonth($date, $url, 2);
    print "</TD>";

    # Advance to the next month
    ++$$date[1];
  }

  print "</TR></TABLE>\n";
}

1;

