#!SED_PERLBIN -w

# Set DEBUG equal to 1 to print debugging information
local ($DEBUG) = 0;
$| = 1 if ($DEBUG == 1);                        # Force prints when debugging

local ($VERSION) = "2.0";			# Current version
local ($COPYRIGHT) = "
Copyright (c) 1999-2003 Xerox Corporation.  All Rights Reserved.

Permission to use, copy, modify  and  distribute  without  charge
this  software,  documentation, images, etc. is granted, provided
that this copyright and the author's name is retained.
    
A fee may be charged for this program ONLY to recover  costs  for
distribution  (i.e.  media costs).  No profit can be made on this
program.
        
The author assumes no responsibility for  disasters  (natural  or
otherwise) as a consequence of use of this software.
            
Adam Stein (adam\@scan.mc.xerox.com)
";

use vars qw($COPYRIGHT $VERSION);

# User changeable values
use constant FETCHWIN => 0;			# Flag indicating to show
						# the "Fetching..." window

# If we don't have the CSA module, punt to using the dtcm_lookup program
eval 'use Calendar::CSA';

if ($@) {
  $USECSA = 0;
} else {
  $USECSA = 1;
}

use CGI qw(:standard);
use CGI::Carp qw(fatalsToBrowser warningsToBrowser);
use Date::Calc qw(Add_Delta_Days Add_Delta_DHMS Add_Delta_YMD Date_to_Text_Long Day_of_Week Day_of_Week_to_Text Days_in_Month Decode_Date_US Monday_of_Week Month_to_Text Parse_Date Today Week_of_Year Week_Number);
if ($USECSA) {
  use Time::Timezone;				# Only needed for CSA
}

# Include other needed Perl files
BEGIN {
  my ($LIBDIR) = "SED_LIBDIR";               # Dir containing *.pl files

  # Add location of additional Perl files
  push(@INC, (($DEBUG) ? "/home/adam/Src/wcal-2.0" : $LIBDIR));

  require "constants.pl";
}

require "calendars.pl";
require "f2.pl";
require "f3.pl";
require "f4.pl";
require "cde.pl";
require "view.pl";
require "month.pl";
require "showlist.pl";
require "addappt.pl";
require "delete.pl";
require "utils.pl" if ($DEBUG);

# Global variables
local ($program);				# Name of current program
local ($mode);					# Insertion/deletion mode

MAIN:
{
  my ($cgi),					# CGI object
  my ($key),					# Key of calendar to display
  my ($path_info),				# Frame path
  my (@today),					# Today's date
  my ($view) = V_MONTH;				# View to show

  # Set name of current program
  ($program = $0) =~ s#.*/##;

  $cgi = new CGI;

  # Tell the browser what type of file we're creating (and to not cache)
  print $cgi->header(-expires => "now");

  warningsToBrowser(1);

  # Figure out what we need to do
  if (!$cgi->param || !defined($cgi->param("key"))) {
    &ShowCalList($cgi);
  } else {
    # Figure out where in the frame chain we are
    $path_info = $cgi->path_info;

    # Create all the frames (if necessary)
    if (!$path_info) {
      &CreateFrameset($cgi, $cgi->param("key"), $view);
      exit(0);
    } elsif ($path_info =~ /f1/) {
      &CreateSideFrameset($cgi, $cgi->param("key"), $view);
      exit(0);
    }

    # Determine the access rights to this calendar
    $mode = &AccessRights($cgi);

    # View to display calendars
    $view = $cgi->param("view") if (defined($cgi->param("view")));

    # Figure out what day to show
    @today = Today;

    # Check if given the date in pieces
    $today[0] = $cgi->param("year") if (defined($cgi->param("year")));
    $today[1] = $cgi->param("month") if (defined($cgi->param("month")));
    $today[2] = $cgi->param("day") if (defined($cgi->param("day")));

    # Check if given the date as a string
    if (defined($cgi->param("date"))) {
      if (!(@today = Decode_Date_US($cgi->param("date"))) &&
	  !(@today = Parse_Date($cgi->param("date")))) {
	die "$program: given an invalid date [", $cgi->param("date"), "]\n";
      }
    }

    if ($cgi->param("action") eq "addform") {
      # Put up "add appointment" form
      &AddApptForm($cgi, $view, \@today);
    } elsif ($cgi->param("action") eq "addappt") {
      # Add appointment to calendar
      &AddAppt($cgi);
    } elsif ($cgi->param("action") eq "deleteappt") {
      # Delete appointment
      &DeleteAppt($cgi, $cgi->param("scope"));
    } else {
      # Fill up each frame
      &ShowFrame2($cgi, $view, \@today, $cgi->param("key"))  # Big calendar
	if ($path_info =~ /f2/);

      &ShowFrame3($cgi, \@today)			# Small calendars
	if ($path_info =~ /f3/);

      &ShowFrame4($cgi, $view, \@today)			# Commands
	if ($path_info =~ /f4/);
    }
  }

  &HTMLFooter($cgi);
}

# Print out an HTML page header
sub HTMLHeader {
  my ($cgi) = shift,				# CGI object
  my ($frame) = shift,				# Frame to display content in
  my ($key) = shift,				# Key of calendar to display
  my ($js),					# JavaScript code
  my ($url);					# URL of this page

  # Create the parameter list in URL format
  foreach ($cgi->param) {
   $params .= "&" if (defined($params));
   $params .= "$_=" . $cgi->param($_);
  }

  # Create the total URL
  $url = $cgi->url . "/$frame\?$params";

# JavaScript to update all the frames at once
$js = <<EOF;
function update(frame, url, dest) {
  // Fix a problem where the CGI.pm package returns an extra "http:..."
  // in the URL when MS Internet Explorer is used.  This only seems to
  // happen with some people but not others.  I blame Microsoft for this
  // (works fine for Netscape -- including the Windows version).  If the
  // URL doesn't have a problem, replace() won't change anything.
  url = url.replace(/^http.*http/, "http");

  // The location of the frames are different relative to the frame
  // this function was called from
  if (frame == 2) {
    // Load up only those frames requested to be changed
    //	(bit 0 = frame 2; bit 1 = frame 3; bit 2 = frame 4)
    if (dest & 0x01) {
      parent.f2.location = url.replace("?", "/f2?");
    }

    if (dest & 0x02) {
      parent.f1.f3.location = url.replace("?", "/f3?");
    }

    if (dest & 0x04) {
      parent.f1.f4.location = url.replace("?", "/f4?");
    }
  } else {
    // Load up only those frames requested to be changed
    //	(bit 0 = frame 2; bit 1 = frame 3; bit 2 = frame 4)
    if (dest & 0x01) {
      parent.parent.f2.location = url.replace("?", "/f2?");
    }

    if (dest & 0x02) {
      parent.f3.location = url.replace("?", "/f3?");
    }

    if (dest & 0x04) {
      parent.f4.location = url.replace("?", "/f4?");
    }
  }
}
EOF

if ($frame =~ /f2/) {
# JavaScript to update the time fields in the 'add appointment' page
$js .= <<EOF;
function changeHour(what, newhour) {
  var hrval = newhour[newhour.selectedIndex].value;

  /*Set new time in text field*/
  if (what == 0) {
    var oldmin = newhour.form.startmin;
    var minval = oldmin[oldmin.selectedIndex].value;

    newhour.form.start.value = hrval + ":" + minval;

    /*Auto calculate what the ending time should be*/
    var nexthour = eval(hrval) + 1;
    var turnover;

    /*Roll over if necessary*/
    if (nexthour > 12) {
      nexthour = 1;
      turnover = 1;
    } else turnover = 0;

    /*Set this before we make nexthour a string*/
    newhour.form.endhr.selectedIndex = nexthour - 1;

    /*Make the ending hour a 2 digit number*/
    if (nexthour < 10) nexthour = "0" + nexthour;

    /*Set ampm setting of the next hour*/
    if (turnover) {
      if (newhour.form.startampm[0].checked == true)
	newhour.form.endampm[1].checked = true;
      else
	newhour.form.endampm[0].checked = true;
    } else {
      newhour.form.endampm[0].checked = newhour.form.startampm[0].checked;
      newhour.form.endampm[1].checked = newhour.form.startampm[1].checked;
    }

    newhour.form.end.value = nexthour + ":" + minval;
  } else {
    var oldmin = newhour.form.endmin;
    var minval = oldmin[oldmin.selectedIndex].value;

    newhour.form.end.value = hrval + ":" + minval;
  }
}

function changeMin(what, newmin) {
  var minval = newmin[newmin.selectedIndex].value;

  /*Set new time in text field*/
  if (what == 0) {
    var oldhr = newmin.form.starthr;
    var oldhr2 = newmin.form.endhr;
    var hrval = oldhr[oldhr.selectedIndex].value;
    var hrval2 = oldhr2[oldhr2.selectedIndex].value;

    newmin.form.start.value = hrval + ":" + minval;
    newmin.form.end.value = hrval2 + ":" + minval;
  } else {
    var oldhr = newmin.form.endhr;
    var hrval = oldhr[oldhr.selectedIndex].value;

    newmin.form.end.value = hrval + ":" + minval;
  }
}

function deleteWin(url, key, id, repeat, perm) {
  if (repeat && (perm & 0x02)) {
    var deleteMsg = window.open("", "deleteWindow", "");

    deleteMsg.focus();
    deleteMsg.resizeTo(300, 150);
    deleteMsg.moveTo(300, 400);

    deleteMsg.document.write("<HTML>");
    deleteMsg.document.write("<HEAD>");
    deleteMsg.document.write("<TITLE>Appointment Editor - Delete</TITLE>");
    deleteMsg.document.write("</HEAD>");
    deleteMsg.document.write("<BODY>");
    deleteMsg.document.write("<P>");
    deleteMsg.document.write("This appointment is a part of a repeating series.");
    deleteMsg.document.write("</P>");

    deleteMsg.document.write("<P>");
    deleteMsg.document.write("Do you want to delete ...?");
    deleteMsg.document.write("</P>");

    deleteMsg.document.write("<HR>");

    deleteMsg.document.write("<CENTER>");
    deleteMsg.document.write('<FORM ACTION="' + url + '">');
    deleteMsg.document.write('<INPUT NAME="action" VALUE="deleteappt" TYPE=hidden>');
    deleteMsg.document.write('<INPUT NAME="key" VALUE="' + key + '" TYPE=hidden>');
    deleteMsg.document.write('<INPUT NAME="id" VALUE="' + id + '" TYPE=hidden>');
    deleteMsg.document.write('<INPUT NAME="scope" TYPE=hidden>');
    deleteMsg.document.write('<INPUT NAME="Cancel" VALUE="Cancel" TYPE=submit onClick="document.forms[0].scope.value = this.name">');
    deleteMsg.document.write('<INPUT NAME="ONE" VALUE="This One Only" TYPE=submit onClick="document.forms[0].scope.value = this.name">');
    deleteMsg.document.write('<INPUT NAME="FORWARD" VALUE="Forward" TYPE=submit onClick="document.forms[0].scope.value = this.name">');
    deleteMsg.document.write('<INPUT NAME="ALL" VALUE="All" TYPE=submit onClick="document.forms[0].scope.value = this.name">');
    deleteMsg.document.write("</FORM>");
    deleteMsg.document.write("</CENTER>");
    deleteMsg.document.write("</BODY></HTML>");
  } else {
    location = url + "?key=" + key + "&action=deleteappt&id=" + id + "&scope=ALL";
  }
}

function repeatWin(obj) {
  // Set number of occurrrences depending on units
  switch (obj[obj.selectedIndex].value) {
    case "One Time":
	document.forms[0].numtimes.value = 1;
	break;
    case "Daily":
	document.forms[0].numtimes.value = 365;
	break;
    case "Weekly":
        document.forms[0].numtimes.value = 52;
        break;
    case "Every Two Weeks":
        document.forms[0].numtimes.value = 26;
        break;
    case "Monthly By Date":
        document.forms[0].numtimes.value = 12;
        break;
    case "Monthly By Weekday":
        document.forms[0].numtimes.value = 12;
        break;
    case "Yearly":
        document.forms[0].numtimes.value = 2;
        break;
    case "Monday Thru Friday":
        document.forms[0].numtimes.value = 52;
        break;
    case "Mon, Wed, Fri":
        document.forms[0].numtimes.value = 52;
        break;
    case "Tuesday, Thursday":
        document.forms[0].numtimes.value = 52;
        break;
    case "Repeat Every ...":
        // Popup a window asking the user the repeat number
        // if we have the "Repeat Every ..." choice
        var repeatMsg = window.open("", "repeatWindow", "");

        repeatMsg.focus();
        repeatMsg.resizeTo(250, 100);
        repeatMsg.moveTo(200, 400);

        repeatMsg.document.write("<HTML>");
        repeatMsg.document.write("<HEAD>");
        repeatMsg.document.write("<TITLE>Repeat Every</TITLE>");
        repeatMsg.document.write("</HEAD>");
        repeatMsg.document.write("<BODY>");
        repeatMsg.document.write("<CENTER>");
        repeatMsg.document.write('<FORM NAME="repeat" onSubmit="javascript:opener.dismissRepeatWin(document.forms[0].repeat.value, document.forms[0].units.selectedIndex); self.close()">');
        repeatMsg.document.write('Repeat Every: <INPUT NAME="repeat" SIZE="4">');
        repeatMsg.document.write('<SELECT NAME="units">');
        repeatMsg.document.write('<OPTION VALUE="Days">Days</OPTION>');
        repeatMsg.document.write('<OPTION VALUE="Weeks">Weeks</OPTION>');
        repeatMsg.document.write('<OPTION VALUE="Months">Months</OPTION>');
        repeatMsg.document.write("</SELECT>");
        repeatMsg.document.write("<BR>");
        repeatMsg.document.write('<INPUT NAME="Apply" VALUE="Apply" TYPE=submit>');
        repeatMsg.document.write('<INPUT NAME="Close" VALUE="Close" TYPE=reset onClick="self.close()">');
        repeatMsg.document.write("</FORM>");
        repeatMsg.document.write("</CENTER>");
        repeatMsg.document.write("</BODY></HTML>");

        break;
  }
}

function dismissRepeatWin(repeat, units) {
  if (repeat == "") {
    alert("The Repeat Every value must be an integer with no sign.");
  } else {
    document.forms[0].repeatval.value = repeat;
    document.forms[0].repeatunits.value = units;

    // Set number of occurrrences depending on units
    switch (document.forms[0].repeatunits.value) {
      case 0:	// Days
        document.forms[0].numtimes.value = 365;
	break;
      case 1:	// Weeks
	document.forms[0].numtimes.value = 52;
	break;
      case 2:	// Months
	document.forms[0].numtimes.value = 12;
	break;
    }
  }
}
EOF
}

# JavaScript to allow a user to type in a date
if ($frame =~ /f4/) {
$js .= <<EOF;
function enterDate(url) {
  // Set up to replace punctuation characters with their hex equivalent
  var ascii = new Array(29)
  var hex = new Array(29)

  ascii[0] = "%"; hex[0] = "%25";	// % MUST be first
  ascii[1] = "!"; hex[1] = "%21";
  ascii[2] = "@"; hex[2] = "%40";
  ascii[3] = "#"; hex[3] = "%23";
  ascii[4] = "\$"; hex[4] = "%24";
  ascii[5] = "^"; hex[5] = "%5E";
  ascii[6] = "&"; hex[6] = "%26";
  ascii[7] = "("; hex[7] = "%28";
  ascii[8] = ")"; hex[8] = "%29";
  ascii[9] = "="; hex[9] = "%3D";
  ascii[10] = "\\\\"; hex[10] = "%5C";
  ascii[11] = "`"; hex[11] = "%60";
  ascii[12] = "+"; hex[12] = "%2B";
  ascii[13] = "|"; hex[13] = "%7C";
  ascii[14] = "~"; hex[14] = "%7E";
  ascii[15] = "["; hex[15] = "%5B";
  ascii[16] = "]"; hex[16] = "%5D";
  ascii[17] = "{"; hex[17] = "%7B";
  ascii[18] = "}"; hex[18] = "%7D";
  ascii[19] = ";"; hex[19] = "%3B";
  ascii[20] = "'"; hex[20] = "%27";
  ascii[21] = ":"; hex[21] = "%3A";
  ascii[22] = '"'; hex[22] = "%22";
  ascii[23] = ","; hex[23] = "%2C";
  ascii[24] = "/"; hex[24] = "%2F";
  ascii[25] = "<"; hex[25] = "%3C";
  ascii[26] = ">"; hex[26] = "%3E";
  ascii[27] = "?"; hex[27] = "%3F";
  ascii[28] = " "; hex[28] = "+";	// This MUST go after the "+" change

  // Get the date from the user
  var date = prompt("Enter the date you wish to see (english syntax OK)", "");

  if (date == "") {
    alert("You must enter a date");
  } else if (date != null) {
    // Change punctuation characters as needed to follow the CGI spec
    for(var loop = 0;loop < 29;++loop) {
      var re = new RegExp("\\\\" + ascii[loop], "g");

      date = date.replace(re, hex[loop]);
    }

    // Concatenate all the parts into a single URL
    var nurl = url + "&date=" + date;

    update(4, nurl, 7);
  }
}
EOF
}

  if ($frame =~ /f[234]/) {
    print $cgi->start_html({script => "$js"}, $key);
  } else {
    print $cgi->start_html($key);
  }
}

# Print out the closing HTML tags
sub HTMLFooter {
  my ($cgi) = shift;				# CGI object

  print $cgi->end_html;
}

# Create the initial frameset consisting of Frame 1 & Frame 2
#
# Frame 1 (f1) will contain 2 frames: f3 & f4
# Frame 2 (f2) will show the calendar with appointments
# Frame 3 (f3) will show the previous and next months
# Frame 4 (f4) will show commands (such as changing the view)
sub CreateFrameset {
  my ($cgi) = shift,				# CGI object
  my ($key) = shift,				# Key of calendar to display
  my ($view) = shift,				# View to show calendar in
  my ($args),					# Args to send in URL
  my ($caladdr),                        	# Calendar addr to connect to
  my ($script_name);				# Name of this program

  # See if we were given a full calendar address or
  # if we need to look it up
  $caladdr = (($key =~ /@/) ? $key : $calendar{$key}[0]);

  $script_name = $cgi->script_name;

  # Little JavaScript to ensure that we can never open
  # a calendar within a frame (it will remove the frames first).
  # Otherwise, we could get the 3 frames *within* frame 2.
  print "<HTML><HEAD><TITLE>$caladdr</TITLE>\n",
	"<SCRIPT LANGUAGE=\"JavaScript1.2\">\n",
	"<!--\n",
	"if (top.frames.length != 0)\n",
	"  top.location=self.document.location;",
	"// -->\n",
	"</SCRIPT>\n",
  	"</HEAD>\n";

  # Args to send in each URL
  $args = "key=$key&view=$view";

  print $cgi->frameset({cols => "250,*", border => 4},
		       $cgi->frame({src => "$script_name/f1?$args",
				    name => "f1", marginwidth => 0,
				    marginheight => 0}),
		       $cgi->frame({src => "$script_name/f2?$args",
				    name => "f2", marginwidth => 0,
				    marginheight => 0}));

  print "</HTML>\n";
}

# Create the side frameset consisting of Frame 3 & Frame 4
#
# Frame 3 (f3) will show the previous and next months
# Frame 4 (f4) will show commands (such as changing the view)
sub CreateSideFrameset {
  my ($cgi) = shift,				# CGI object
  my ($key) = shift,				# Key of calendar to display
  my ($view) = shift,				# View to show calendar in
  my ($args),					# Args to send in URL
  my ($caladdr),                        	# Calendar addr to connect to
  my ($script_name);				# Name of this program

  # See if we were given a full calendar address or
  # if we need to look it up
  $caladdr = (($key =~ /@/) ? $key : $calendar{$key}[0]);

  $script_name = $cgi->script_name;

  print "<HTML><HEAD><TITLE>$caladdr</TITLE></HEAD>\n";

  # Args to send in each URL
  $args = "key=$key&view=$view";

  print $cgi->frameset({rows => "400,*", border => 2},
		       $cgi->frame({src => "$script_name/f3?$args",
				    name => "f3"}),
		       $cgi->frame({src => "$script_name/f4?$args",
				    name => "f4"}));
}

